@@ -2096,9 +2096,9 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 	 * without as a last resort. This can happen due to
 	 * permissions.
 	 */
-	fd = open (filename, O_RDONLY | O_NOATIME);
-	if (fd == -1) {
-		fd = open (filename, O_RDONLY);
+	fd = g_open (filename, O_RDONLY | O_NOATIME);
+	if (fd == -1 && errno == EPERM) {
+		fd = g_open (filename, O_RDONLY);
 
 		if (fd == -1) {
 			return FALSE;
