@@ -21,13 +21,25 @@
 
 #include "config.h"
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
+#include <errno.h>
+#include <fcntl.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <sys/mman.h>
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/poppler.h>
 
 #include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-utils.h>
+#include <libtracker-common/tracker-file-utils.h>
 
 #include <libtracker-extract/tracker-extract.h>
 
@@ -265,29 +277,82 @@ write_pdf_data (PDFData               data,
 }
 
 G_MODULE_EXPORT gboolean
-tracker_extract_get_metadata (const gchar          *uri,
-                              const gchar          *mimetype,
-                              TrackerSparqlBuilder *preupdate,
-                              TrackerSparqlBuilder *metadata,
-                              GString              *where)
+tracker_extract_get_metadata (TrackerExtractInfo *info)
 {
 	TrackerConfig *config;
 	GTime creation_date;
 	GError *error = NULL;
+	TrackerSparqlBuilder *metadata, *preupdate;
+	const gchar *graph;
 	TrackerXmpData *xd = NULL;
 	PDFData pd = { 0 }; /* actual data */
 	PDFData md = { 0 }; /* for merging */
 	PopplerDocument *document;
 	gchar *xml = NULL;
-	gchar *content;
+	gchar *content, *uri;
 	guint n_bytes;
 	GPtrArray *keywords;
+	GString *where;
 	guint i;
+	GFile *file;
+	gchar *filename;
+	int fd;
+	gchar *contents = NULL;
+	gsize len;
+	struct stat st;
 
 	g_type_init ();
 
-	document = poppler_document_new_from_file (uri, NULL, &error);
+	metadata = tracker_extract_info_get_metadata_builder (info);
+	preupdate = tracker_extract_info_get_preupdate_builder (info);
+	graph = tracker_extract_info_get_graph (info);
 
+	file = tracker_extract_info_get_file (info);
+	filename = g_file_get_path (file);
+
+	fd = g_open (filename, O_RDONLY | O_NOATIME, 0);
+	if (fd == -1 && errno == EPERM) {
+		fd = g_open (filename, O_RDONLY, 0);
+	}
+
+	if (fd == -1) {
+		g_warning ("Could not open pdf file '%s': %s\n",
+		           filename,
+		           g_strerror (errno));
+		g_free (filename);
+		return FALSE;
+	}
+
+	if (fstat (fd, &st) == -1) {
+		g_warning ("Could not fstat pdf file '%s': %s\n",
+		           filename,
+		           g_strerror (errno));
+		close (fd);
+		g_free (filename);
+		return FALSE;
+	}
+
+	if (st.st_size == 0) {
+		contents = NULL;
+		len = 0;
+	} else {
+		contents = (gchar *) mmap (NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
+		if (contents == NULL) {
+			g_warning ("Could not mmap pdf file '%s': %s\n",
+			           filename,
+			           g_strerror (errno));
+			close (fd);
+			g_free (filename);
+			return FALSE;
+		}
+		len = st.st_size;
+	}
+
+	g_free (filename);
+	uri = g_file_get_uri (file);
+
+	document = poppler_document_new_from_data (contents, len, NULL, &error);
+	
 	if (error) {
 		if (error->code == POPPLER_ERROR_ENCRYPTED) {
 			tracker_sparql_builder_predicate (metadata, "a");
@@ -297,6 +362,8 @@ tracker_extract_get_metadata (const gchar          *uri,
 			tracker_sparql_builder_object_boolean (metadata, TRUE);
 
 			g_error_free (error);
+			g_free (uri);
+
 			return TRUE;
 		} else {
 			g_warning ("Couldn't create PopplerDocument from uri:'%s', %s",
@@ -304,6 +371,8 @@ tracker_extract_get_metadata (const gchar          *uri,
 			           error->message ? error->message : "no error given");
 
 			g_error_free (error);
+			g_free (uri);
+
 			return FALSE;
 		}
 	}
@@ -312,6 +381,7 @@ tracker_extract_get_metadata (const gchar          *uri,
 		g_warning ("Could not create PopplerDocument from uri:'%s', "
 		           "NULL returned without an error",
 		           uri);
+		g_free (uri);
 		return FALSE;
 	}
 
@@ -409,6 +479,10 @@ tracker_extract_get_metadata (const gchar          *uri,
 			                                              xd->model ? xd->model : "");
 
 			tracker_sparql_builder_insert_open (preupdate, NULL);
+			if (graph) {
+				tracker_sparql_builder_graph_open (preupdate, graph);
+			}
+
 			tracker_sparql_builder_subject_iri (preupdate, equip_uri);
 			tracker_sparql_builder_predicate (preupdate, "a");
 			tracker_sparql_builder_object (preupdate, "nfo:Equipment");
@@ -422,7 +496,12 @@ tracker_extract_get_metadata (const gchar          *uri,
 				tracker_sparql_builder_predicate (preupdate, "nfo:model");
 				tracker_sparql_builder_object_unvalidated (preupdate, xd->model);
 			}
+
+			if (graph) {
+				tracker_sparql_builder_graph_close (preupdate);
+			}
 			tracker_sparql_builder_insert_close (preupdate);
+
 			tracker_sparql_builder_predicate (metadata, "nfo:equipment");
 			tracker_sparql_builder_object_iri (metadata, equip_uri);
 			g_free (equip_uri);
@@ -522,6 +601,10 @@ tracker_extract_get_metadata (const gchar          *uri,
 				tracker_sparql_builder_object_iri (metadata, addruri);			
 
 				tracker_sparql_builder_insert_open (preupdate, NULL);
+				if (graph) {
+					tracker_sparql_builder_graph_open (preupdate, graph);
+				}
+
 				tracker_sparql_builder_subject_iri (preupdate, addruri);
 
 				g_free (addruri);
@@ -549,6 +632,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 				        tracker_sparql_builder_object_unvalidated (preupdate, xd->country);
 				}
 
+				if (graph) {
+					tracker_sparql_builder_graph_close (preupdate);
+				}
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
@@ -569,6 +655,11 @@ tracker_extract_get_metadata (const gchar          *uri,
 
 			tracker_sparql_builder_object_blank_close (metadata); /* GeoLocation */
 		}
+
+                if (xd->regions) {
+	                tracker_xmp_apply_regions (preupdate, metadata, graph, xd);
+                }
+
 		tracker_xmp_free (xd);
 	} else {
 		/* So if we are here we have NO XMP data and we just
@@ -577,6 +668,8 @@ tracker_extract_get_metadata (const gchar          *uri,
 		write_pdf_data (pd, metadata, keywords);
 	}
 
+	where = g_string_new ("");
+
 	for (i = 0; i < keywords->len; i++) {
 		gchar *p, *escaped, *var;
 
@@ -585,11 +678,26 @@ tracker_extract_get_metadata (const gchar          *uri,
 		var = g_strdup_printf ("tag%d", i + 1);
 
 		/* ensure tag with specified label exists */
+		tracker_sparql_builder_append (preupdate, "INSERT { ");
+
+		if (graph) {
+			tracker_sparql_builder_append (preupdate, "GRAPH <");
+			tracker_sparql_builder_append (preupdate, graph);
+			tracker_sparql_builder_append (preupdate, "> { ");
+		}
+
 		tracker_sparql_builder_append (preupdate,
-		                               "INSERT { _:tag a nao:Tag ; nao:prefLabel \"");
+		                               "_:tag a nao:Tag ; nao:prefLabel \"");
 		tracker_sparql_builder_append (preupdate, escaped);
+		tracker_sparql_builder_append (preupdate, "\"");
+
+		if (graph) {
+			tracker_sparql_builder_append (preupdate, " } ");
+		}
+
+		tracker_sparql_builder_append (preupdate, " }\n");
 		tracker_sparql_builder_append (preupdate,
-		                               "\" }\nWHERE { FILTER (NOT EXISTS { "
+		                               "WHERE { FILTER (NOT EXISTS { "
 		                               "?tag a nao:Tag ; nao:prefLabel \"");
 		tracker_sparql_builder_append (preupdate, escaped);
 		tracker_sparql_builder_append (preupdate,
@@ -607,6 +715,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 	}
 	g_ptr_array_free (keywords, TRUE);
 
+	tracker_extract_info_set_where_clause (info, where->str);
+	g_string_free (where, TRUE);
+
 	tracker_sparql_builder_predicate (metadata, "nfo:pageCount");
 	tracker_sparql_builder_object_int64 (metadata, poppler_document_get_n_pages (document));
 
@@ -629,8 +740,15 @@ tracker_extract_get_metadata (const gchar          *uri,
 	g_free (pd.creation_date);
 	g_free (pd.author);
 	g_free (pd.date);
+	g_free (uri);
 
 	g_object_unref (document);
 
+	if (contents) {
+		munmap (contents, len);
+	}
+
+	close (fd);
+
 	return TRUE;
 }
